const urlParams = new URLSearchParams(window.location.search);

function change_modal_info(data) {
    const modalId = "#editDomainRecord";
    const modal = document.querySelector(modalId);
    const keys = Object.keys(data);
    
    modal.querySelectorAll("[name]").forEach(input => {
        if (keys.includes(input.name)) {
            const inputValue = data[input.name];
            if (inputValue !== null && isNaN(inputValue)) {
                const domainSuffix = `.${urlParams.get("domain")}`;
                if (input.name === "name" && inputValue.endsWith(domainSuffix)) {
                    input.value = inputValue.slice(0, -domainSuffix.length);
                }
                if (inputValue.endsWith(".")) {
                    input.value = inputValue.slice(0, -1);
                }
                input.value = inputValue;
            } else {
                input.value = inputValue;
            }
        }        
    });
    modal.querySelector("#edit-record").addEventListener("click", event => domainRecord(event, 'edit', {id: data["id"]}));
    $(modal).modal('show');
    domainRecord_values(document.querySelector("#editDomainRecord"));
}

function add_firewall_rule(form) {
    const protocol = form.querySelector('#protocol').value;
    let ports = form.querySelector('#ports').value;
    let selectedAddresses = $('#addresses').find(':selected').map(function () {
        return $(this).text();
    }).get();

    if (protocol !== 'tcp' && protocol !== 'udp') {
        form.querySelector('#protocol').value = 'tcp';
    }

    const portPattern = /^\d+(?:-\d+)?$/;
    if (!portPattern.test(ports)) {
        form.querySelector('#ports').classList.add('is-invalid');
        return;
    }
    if (ports.indexOf('-') !== -1) {
        const [startPort, endPort] = ports.split('-');
        if (Number(endPort) < Number(startPort)) {
            form.querySelector('#ports').classList.add('is-invalid');
            return;
        }
    }

    const addressPattern = /^(?:\d{1,3}\.){3}\d{1,3}(?:\/\d{1,2})?$/;

    selectedAddresses = selectedAddresses.map(address => {
        if (address.trim() === 'Todos IPV6') {
            return '::/0';
        } else if (address.trim() === 'Todos IPV4') {
            return '0.0.0.0/0';
        } else if (addressPattern.test(address.trim())) {
            return address.trim();
        }
        return null;
    }).filter(Boolean);

    if (selectedAddresses.length === 0) {
        form.querySelector('#addresses').classList.add('is-invalid');
        return;
    }

    const firewallRule = {
        protocol: protocol,
        ports: ports,
        addresses: selectedAddresses
    };

    const urlParams = new URLSearchParams(window.location.search);
    const queryParams = new URLSearchParams(firewallRule).toString();
    const requestURL = `modulos/servidores/comandoCliente.php?funcao=acao_gerenciar_firewall&PID=${urlParams.get('ID')}&${queryParams}&action=add`;
    window.location.href = requestURL;
}

function domainRecord(event, action, obj = {}) {
    event.preventDefault();
    const form = event.target.closest('.modal').querySelector('form');
    const type = form.querySelector('#type').value;
    const dataInput = form.querySelector('#data');
    const nameInput = form.querySelector('#name');
    const priorityInput = form.querySelector('#priority');
    const portInput = form.querySelector('#port');
    const weightInput = form.querySelector('#weight');
    const ttlInput = form.querySelector('#ttl');
    const flagInput = form.querySelector('#flag');
    const tagInput = form.querySelector('#tag');

    const ipAddressRegex = /^(\d{1,3}\.){3}\d{1,3}$/;
    const ipv6Regex = /^[0-9a-fA-F:]+$/;
    const domainNameRegex = /^([a-zA-Z0-9.-]+\.)*[a-zA-Z0-9-]+\.[a-zA-Z]{2,}$/;
    const srvRegex = /^_([a-zA-Z0-9-]+)\._([a-zA-Z0-9-]+)$/;
    const allowedTagValues = ['issue', 'issuewild', 'iodef'];
    const isValidTag = allowedTagValues.includes(tagInput.value);
    
    const priority = parseInt(priorityInput.value);
    const port = parseInt(portInput.value);
    const weight = parseInt(weightInput.value);
    const ttl = parseInt(ttlInput.value);
    const flag = parseInt(flagInput.value);

    const isValidData = (type === 'A' && ipAddressRegex.test(dataInput.value)) ||
    (type === 'AAAA' && ipv6Regex.test(dataInput.value)) ||
    (type === 'CNAME' && domainNameRegex.test(dataInput.value)) ||
    (type === 'MX' && domainNameRegex.test(dataInput.value) && !isNaN(priority) && priority >= 0) ||
    (type === 'SRV' && srvRegex.test(nameInput.value) && !isNaN(port) && !isNaN(weight)) ||
    (type === 'CAA' && !isNaN(flag) && flag >= 0 && flag <= 255) ||
    (type === 'TXT' && dataInput.value.trim() !== '') ||
    (type === 'NS' && domainNameRegex.test(dataInput.value));

    const isValidHostname = (type === "SRV") ? srvRegex.test(nameInput.value) : typeof nameInput.value === "string";

    dataInput.classList.toggle('is-invalid', !isValidData);
    nameInput.classList.toggle('is-invalid', !isValidHostname);
    priorityInput.classList.toggle('is-invalid', isNaN(priority) || !isValidData);
    portInput.classList.toggle('is-invalid', isNaN(port) || !isValidData);
    weightInput.classList.toggle('is-invalid', isNaN(weight) || !isValidData);
    ttlInput.classList.toggle('is-invalid', isNaN(ttl));
    flagInput.classList.toggle('is-invalid', isNaN(flag) || !isValidData);
    tagInput.classList.toggle('is-invalid', !isValidTag);
    
    if (!isValidData || !isValidHostname || (type === "CAA" && !isValidTag)) {
        return;
    }

    switch(type) {
        case "CNAME":
        case "MX":
        case "NS":
        case "CAA":
            if (dataInput.value.endsWith(".")) {
                dataInput.value = dataInput.value.slice(0, -1);
            }
            dataInput.value += ".";
            break;
    }    

    const domainRecord = {
        type: type,
        name: nameInput.value,
        data: dataInput.value,
        priority: isNaN(priority) ? null : priority,
        port: isNaN(port) ? null : port,
        ttl: isNaN(ttl) ? null : ttl,
        weight: isNaN(weight) ? null : weight,
        flag: isNaN(flag) ? null : flag,
        tag: tagInput.value,
    };

    const queryParams = new URLSearchParams(domainRecord).toString();
    const requestURL = (action === "add") ? 
    `modulos/servidores/comandoCliente.php?funcao=acao_dominio_record&domain=${urlParams.get('domain')}&PID=${urlParams.get('ID')}&${queryParams}&action=add` : 
    `modulos/servidores/comandoCliente.php?funcao=acao_dominio_record&domain=${urlParams.get('domain')}&PID=${urlParams.get('ID')}&${queryParams}&record_id=${obj['id']}&action=edit`;
    window.location.href = requestURL;

}

function toggleRecordFields(form) {
    const descriptionMap = {
        A: "Endereço IP no formato xxx.xxx.xxx.xxx",
        AAAA: "Endereço IPv6 no formato: xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx",
        CNAME: "Nome de domínio no formato: example.com",
        MX: "Nome de domínio do servidor de email no formato: example.com",
        TXT: "Texto ou informação adicional",
        NS: "Nome de domínio do servidor de nomes no formato: ns.example.com",
        SRV: "Nome de domínio no formato: example.com",
        CAA: "Informações de autorização do certificado",
    };
    const descriptionTextMap = {
        A: "Irã direcionar para",
        AAAA: "Irã direcionar para",
        CNAME: "É um alias de",
        MX: "Provedores de e-mail servidor de e-mail",
        TXT: "Valor",
        NS: "Irã direcionar para",
        SRV: "Irã direcionar para",
        CAA: "Autoridade concedida para",
    };
    const defaultTTLMap = {
        A: 3600,
        AAAA: 3600,
        CNAME: 43200,
        MX: 14400,
        TXT: 3600,
        NS: 86400,
        SRV: 43200,
        CAA: 3600,
    };    
    const type = form.querySelector('[name="type"]').value;

    const nameField = form.querySelector('#name-desc');
    const descriptionField = form.querySelector('#description-desc');
    descriptionField.textContent = descriptionMap[type] || '';
    const dataLabel = form.querySelector('label[for="data"]');
    const capitalizedtype = type.toUpperCase();
    dataLabel.textContent = `${descriptionTextMap[capitalizedtype]}:`;

    nameField.classList[(type !== "SRV") ? "add" : "remove"]("d-none");

    const priorityField = form.querySelector('[name="priority"]').parentNode;
    const portField = form.querySelector('[name="port"]').parentNode;
    const weightField = form.querySelector('[name="weight"]').parentNode;
    const ttlField = form.querySelector('[name="ttl"]').parentNode;
    const flagField = form.querySelector('[name="flag"]').parentNode;
    const tagField = form.querySelector('[name="tag"]').parentNode;

    const allFields = [priorityField, portField, weightField, ttlField, flagField, tagField];
    allFields.forEach(field => {
        field.classList.add('d-none');
        const input = field.querySelector('input, select');
        if (input) {
            input.disabled = true;
            input.value = '';
        }
    });

    if (type === 'MX' || type === 'SRV') {
        priorityField.classList.remove('d-none');
        priorityField.querySelector('input').disabled = false;
    }
    if (type === 'SRV') {
        portField.classList.remove('d-none');
        weightField.classList.remove('d-none');
        portField.querySelector('input').disabled = false;
        weightField.querySelector('input').disabled = false;
    }
    if (type === 'CAA') {
        flagField.classList.remove('d-none');
        tagField.classList.remove('d-none');
        flagField.querySelector('input').disabled = false;
        tagField.querySelector('select').disabled = false;
    }
    ttlField.classList.remove('d-none');
    ttlField.querySelector('input').disabled = false;
    ttlField.querySelector('input').value = defaultTTLMap[capitalizedtype];
}

function domainRecord_values(modal) {
    const form = modal.querySelector("form");
    const select = form.querySelector('[name="type"]');
    select.addEventListener('change', e=> toggleRecordFields(form));
    modal.addEventListener('show.bs.modal', toggleRecordFields(form));
}

$(document).ready(function() {
    $('.custom-select').select2({
      tags: true,
      tokenSeparators: [',', ' '],
      placeholder: 'Insira os endereços de IPs',
      theme: "bootstrap"
    });
    document.querySelectorAll('form span.domain-name').forEach(domainName => domainName.innerHTML = urlParams.get("domain"));
    domainRecord_values(document.querySelector("#createDomainRecord"));
});